# https://cryptography.io/en/latest/hazmat/primitives/symmetric-encryption
import math
import base64
import logging
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import (
Cipher, algorithms, modes
)

from . import cipher, cipher_utils

class Encryptor(cipher.Encryptor):
    """Encryptor class for symmetric encryption through Fernet module.
    Essential parameters such as generated keys and cipher modes 
    are dunder-prefixed for reuse while decrypting the message.
    """
    def __init__(self, cipher_params: dict, arg_params: dict, **kwargs):
        """Initializer for primitives and underlying cipher objects

        :param kwargs: Consolidation of parameters consumed by base cipher library
        :type dict
        """
        self._algorithm = cipher_params.get("algorithm", "AES")
        self._mode = cipher_params.get("mode", "CBC")
        self.__key = arg_params.get("key", None) or cipher_utils.gen_random_secret(arg_params.get("key_size", 32)).encode('utf-8')
        self.__iv = arg_params.get("iv", None) or cipher_utils.gen_random_secret(arg_params.get("iv_size", 16)).encode('utf-8')
        self.__encoding = arg_params.get("encoding", "utf-8")
        self.__pad_len = 0
        self.__algorithm = arg_params.get("algorithm", {})
        self.__mode = arg_params.get("mode", {})
        if self._algorithm not in CIPHER_ARGS['algorithms']:
            raise Exception("Unsupported algo type {}".format(self._algorithm))
        if self._mode not in CIPHER_ARGS['modes']:
            raise Exception("Unsupported mode type {}".format(self._mode))
        algo_obj = vars(algorithms)[self._algorithm]
        mode_obj = vars(modes)[self._mode]
        self._cipher = Cipher(
                algo_obj(self.__key, **self.__algorithm),
                mode_obj(self.__iv, **self.__mode),
                default_backend()
            )

    def encrypt(self, plain_text):
        """Template method for encryption of a single message
        
        :param plain_text: input message for encryption
        :type str
        :returns: encrypted output for secure storage
        :rtype str
        """
        l = len(plain_text)
        self.__pad_len = (1<<(int(math.log2(l)) + 3)) - l
        ptext_padded =  plain_text + \
            cipher_utils.gen_random_secret(self.__pad_len)
        enc = self._cipher.encryptor()
        cipher_text = enc.update(ptext_padded.encode(self.__encoding)) \
                      + enc.finalize()
        return cipher_text

class Decryptor(cipher.Decryptor):
    """Decryptor class for symmetric decryption through Fernet module.
    Essential parameters such as generated keys and cipher modes 
    are dunder-prefixed for reuse while decrypting the message.
    """
    def __init__(self, cipher_params: dict, arg_params: dict, **kwargs):
        """Initializer for primitives and underlying cipher objects

        :param kwargs: Consolidation of parameters consumed by base cipher library
        :type dict
        """
        self.__key = arg_params.get("key", None)
        self.__iv = arg_params.get("iv", None)
        self.__encoding = arg_params.get("encoding", "utf-8")
        self.__pad_len = arg_params.get("pad_len", 0)
        if any([self.__key, self.__iv]) is None:
            raise Exception("invalid seed params provided for decryption")
        self._algorithm = cipher_params.get("algorithm", "AES")
        self._mode = cipher_params.get("mode", "CBC")
        self.__algorithm = arg_params.get("algorithm", {})
        self.__mode = arg_params.get("mode", {})
        if self._algorithm not in CIPHER_ARGS['algorithms']:
            raise Exception("Unsupported algo type {}".format(self._algorithm))
        if self._mode not in CIPHER_ARGS['modes']:
            raise Exception("Unsupported mode type {}".format(self._mode))
        algo_obj = eval("algorithms.{}".format(self._algorithm))
        mode_obj = eval("modes.{}".format(self._mode))
        self._cipher = Cipher(
                algo_obj(self.__key, **self.__algorithm),
                mode_obj(self.__iv, **self.__mode),
                default_backend()
            )

    def decrypt(self, cipher_text):
        """Template method for decryption of a cipher text
        
        :param cipher_text: input message for decryption
        :type str
        :returns: decryption output access/modification
        :rtype str
        """
        dec = self._cipher.decryptor()
        plain_text = dec.update(cipher_text) \
                    + dec.finalize()
        return plain_text.decode(self.__encoding)[:-1*self.__pad_len]

if __name__ != "__main__":
    CIPHER_PKG = {'algorithms': algorithms, 'modes': modes}
    CIPHER_ARGS = {k:cipher_utils.expand_attrs(v) \
                    for k,v in CIPHER_PKG.items()}



import os
import base64
import logging
from . import cipher, cipher_utils

import nacl.utils
from nacl.public import PrivateKey, Box

"""Libsodium wrapper using pyNaCl library"""

class Encryptor(cipher.Encryptor):
    """Wrapper for nacl encryption"""
    def __init__(self, cipher_params: dict, arg_params: dict, **kwargs):
        self.__key_size = arg_params.get("key_size", "32")
        self.__encoding = arg_params.get("encoding", "utf-8")
        self._private_key = cipher_params.get("private_key", cipher_utils.gen_random_secret(int(self.__key_size)))
        pkey_obj = PrivateKey(self._private_key.encode(self.__encoding))
        self.cipher = Box(pkey_obj, pkey_obj.public_key)

    def encrypt(self, plain_text: str):
        return self.cipher.encrypt(plain_text.encode('utf-8'))

class Decryptor(cipher.Decryptor):
    """Wrapper for nacl decryption"""
    def __init__(self, cipher_params: dict, arg_params: dict, **kwargs):
        if 'private_key' not in cipher_params:
            raise Exception("Missing private key in cipher param config for decryption.")
        self._private_key = cipher_params["private_key"]
        self.__encoding = arg_params.get("encoding", "utf-8")
        pkey_obj = PrivateKey(self._private_key.encode(self.__encoding))
        public_key = pkey_obj.public_key
        self.cipher = Box(pkey_obj, pkey_obj.public_key)

    def decrypt(self, cipher_text):
        return self.cipher.decrypt(cipher_text).decode(self.__encoding)

if __name__ != "__main__":
    CIPHER_PKG = {}
    CIPHER_ARGS = {}



import re
import os
import stat
import argparse
import ciphers
import core.view
import core.util
import core.config

class CipherFactory():
    """Factory wrapper for dispatching appropriate
    cipher-based encryptor/decryptor
    """
    def __init__(self):
        self._enc = None
        self._dec = None
        self._parser_cfg = {}
        self._cipher_obj = None
        self._cipher_configured = False
        self._parser_configured = False
        self._cfg_obj = core.config.CipherConfig()

    def create_vault(self):
        """Vault creation helper method"""
        if "vault_file" not in self._parser_cfg:
            raise Exception("vault file not specified in CLI")
        vault_path = self._parser_cfg["vault_file"]
        if os.path.exists(vault_path):
            raise Exception("vault file already exists: {}".format(vault_path))
        elif not os.path.isabs(vault_path):
            core.util.safe_write(os.getcwd(), vault_path, self.encryptor.encrypt("[]"))
        else:
            core.util.safe_write("", vault_path, self.encryptor.encrypt("[]"))
        os.chmod(vault_path, 
            stat.S_IRUSR | stat.S_IWUSR | stat.S_ENFMT)
        self.update_cfg_file()

    def load_cmd_cfg(self, parse_obj: argparse.Namespace):
        """Maintains command line arguments"""
        self._parser_cfg = vars(parse_obj)
        self._parser_configured = True
        
    def load_param_cfg(self, config_obj: core.config.CipherConfig):
        """Maintains yaml config arguments"""
        if not self._parser_configured:
            raise Exception("Please provide CLI arguments into factory")
        elif not self._parser_cfg["create_vault"]:
            del self._cfg_obj
        self._cfg_obj = config_obj
        self._cipher_configured = True
    
    def is_requested(self, operation):
        return bool(self._parser_cfg[operation])

    def update_cfg_file(self):
        """Flushes new content into the config file"""
        cfg_attr = vars(self._enc)
        arg_params = {k[12:]:v for k,v in cfg_attr.items() 
                        if bool(re.match("_Encryptor__+", k))}
        cipher_params = {k[1:]:v for k,v in cfg_attr.items() 
                        if bool(re.match("_+", k)) and 
                        not bool(re.match("_Encryptor__+", k))}
        orig_cfg_path = self._parser_cfg.get('cfg_path') or 'cfg.yaml'
        cfg_filename = orig_cfg_path if not os.path.isabs(orig_cfg_path) \
                        else os.path.basename(orig_cfg_path)
        cfg_dirname = os.path.dirname(orig_cfg_path) \
                    if os.path.isabs(orig_cfg_path) else os.getcwd()
        self._cfg_obj.overwrite = self._parser_cfg['overwrite_cfg']
        final_path = self._cfg_obj.store(cfg_filename = cfg_filename,
                                cfg_dirname = cfg_dirname,
                                arg_params = arg_params,
                                cipher_params = cipher_params)
        view_obj = core.view.View()
        view_obj.print("updated cipher configuration flushed to: " + final_path)

    def _load_cipher_module(self):
        if not self._parser_configured:
            raise Exception("Please provide appropriate CLI configs")
        if self._cipher_obj is None:
            self._cipher_obj = __import__("ciphers.{}".format(self._parser_cfg['cipher_suite']), fromlist=[ciphers])

    @property
    def encryptor(self):
        """Returns an encryptor instance"""
        self._load_cipher_module()
        if self._enc is None:
            params = self._cfg_obj()
            cipher_params = params.pop("cipher_params", {})
            arg_params = params.pop("arg_params", {})
            self._enc = self._cipher_obj.Encryptor(cipher_params, arg_params, **params)
            if self._parser_cfg["overwrite_cfg"]:
                self.update_cfg_file()
        return self._enc
    
    @property
    def decryptor(self):
        """Returns a decryptor instance"""
        self._load_cipher_module()
        if self._dec is None:
            self._dec = self._cipher_obj.Decryptor(**self._cfg_obj())
        return self._dec



import json
import core.view
import core.model

class Record:
    """Wrapper for an individual record maintained in the vault file"""
    def __init__(self, model_type: str):
        self._model_type = model_type
        self._model = core.model.Model(model_type)
        self._content = {}
    
    def __call__(self):
        return self._content

    def load(self, content: dict):
        """Populates record content from existing data
        
        :param  content: Indexed record contents
        :type   content: dict
        """
        if self._model.validate_record(content):
            self._content = content
        else:
            raise Exception("the record obtained is invalid")
    
    def create_interactive(self):
        """Populates record content interactively via end user"""
        if bool(self._content):
            raise Exception("creating record: content already exists")
        view_obj = core.view.View()
        view_obj.print("Please enter your details associated with", self._model_type)
        for field in self._model.get_fields():
            self._content[field['name']] = view_obj.prompt_read(field['name'])
        if not self._model.validate_record(self._content):
            raise Exception("details entered for the given record are invalid")
    
    def display(self, mode="table", indent = 1):
        """Displays contents of a record
        
        :param  mode: The display format
        :type   mode: str (json, yaml, table)
        """
        view_obj = core.view.View()
        if mode == "json":
            view_obj.print(json.dumps(self._content, indent = indent))
        elif mode == "yaml":
            view_obj.print(yaml.dump(self._content, indent = indent))
        else:
            for field in self._content:
                view_obj.print("\t {} : {}", field, self._content[field])
    
    def modify_field(self, key: str, val: str):
        """Modifies content of the given field
        
        :param  key
        :type   key: str

        :param  val
        :type   val: str
        """
        if key not in self._content:
            raise Exception("field {} missing in record".format(key))
        self._content[key] = val



import os
import re
import string
import shutil
import pkgutil
import tempfile
import ciphers

def is_valid_suite(suite_name: str):
        """Cipher suite validity checker

        :param  suite_name: Name of the cipher suite
        :type   suite_name: str
        :rtype: bool
        """
        try:
            __import__("ciphers.{}".format(suite_name), fromlist=[ciphers])
            return True
        except ImportError:
            return False

def valid_encr_attr(attr):
    """Filter for arg_param attributes for a cipher suite

    :param  attr: Argument field to be validated for a given cipher
    :type   attr: str
    :rtype: bool
    """
    return attr[:12] == '_Encryptor__'

def copy(src: str, dst: str):
    if os.path.isfile(src):
        shutil.copyfile(src, dst)
    else:
        shutil.copy(src, dst)

def safe_write(directory, name, content, overwrite=False):
        """Writes content in a temporary location and renames/overwrites the file accordingly

        :param  directory: The dir hosting the final file
        :type   directory: str
        :param  name: propogated name of the regular file
        :type   name: str
        :param  content: Data to be written onto the file
        :type   content: str
        :param  overwrite: Overwrite file if existing
        :type   overwrite: bool
        """
        _name_prefix, _name_suffix = os.path.splitext(name)
        if not bool(_name_suffix):
            _name_suffix = ".vault"
        fd,  tmp_path = tempfile.mkstemp(
                        prefix = "{}_".format(_name_prefix),
                        suffix= _name_suffix,
                        dir = directory
                    )
        with os.fdopen(fd, "wb") as tmp_file:
            tmp_file.write(content)
        _final_path = os.path.join(directory, name)
        if overwrite or not os.path.exists(_final_path):
            os.rename(tmp_path, _final_path)
        else:
            _final_path = tmp_path
        return _final_path

def join_path(parent_path, relative_path):
    """Abstracts file path concatenation

    :param  parent_path: The path housing the relative path
    :type   parent_path: str
    :param  relative_path: The current path referenced from the base location
    :type   relative_path: str
    :rtype: str
    """
    return os.path.join(parent_path, relative_path)

def get_abs_path(relative_path):
    """Abstracts absolute path
    
    :param  relative_path: The current path relative to a reference
    :type   relative_path: str
    :rtype: str
    """
    return join_path(os.path.abspath(__file__), relative_path)

def get_models_path():
    return join_path(
        os.path.dirname(os.path.abspath(__file__)), 
        "models")

def is_valid_file(abs_path):
    return os.path.exists(abs_path) and \
            os.path.isfile(abs_path)

def is_valid_dir(abs_path):
    return os.path.exists(abs_path) and \
            os.path.isdir(abs_path)

class CollectionMeta(type):
    """Metaclass for accrual of supported suites and models"""
    def __init__(cls, *args, **kwargs):
        pass

    @property
    def cipher_suites(self):
        cipher_modules = pkgutil.iter_modules(ciphers.__path__)
        cipher_names = map(lambda module: module.name, cipher_modules)
        return filter(lambda suite: suite[:6] != 'cipher', cipher_names)
    
    @property
    def model_collection(self):
        return [ re.sub("models/|.json", "", mdl) \
                for mdl in os.listdir(get_models_path())]

class Schema(type):
    """Metaclass for reserving the schema blueprint"""
    def __init__(self, *args, **kwargs):
        self._field = \
        """
        {
            "name": ${name},
            "type": ${type},
            "required": ${required}
        }
        """
        self._schema = \
        """
        {
            "Title": ${Title},
            "description": ${description},
            "fields": ${fields}
        }
        """
    
    @property
    def model_template(self):
        return string.Template(self._schema)

    @property
    def field_template(self):
        return string.Template(self._field)



from __future__ import absolute_import, division, print_function
import sys
import logging

__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
]

__title__ = "sec-vault"
__summary__ = (
   "SWISS Army-knife password management toolkit"             
)
__uri__ = "https://github.com/siddharths2710/sec-vault"
__version__ = "0.0.1"
__author__ = "Siddharth Srinivasan"
__email__ = "siddharths2710@yahoo.com"
__license__ = "LGPL-2.1 License"
__copyright__ = "Copyright 2021 {0}".format(__author__)


logging.basicConfig(
    format="%(levelname)s %(message)s",
    level=logging.INFO,
    stream=sys.stdout,
)




#! /usr/bin/env python3

import os
import sys
import logging
import traceback
import ciphers
import core.util
import core.view
import core.model
import core.parser
import core.config
import core.record
import core.collection
import core.cipherfactory

def _show_params(vault_args, factory, **kwargs):
    view_obj = core.view.View()
    view_obj.print("------Supported parameter values for {}---------".format(vault_args.cipher_suite) )
    for param in factory.cipher_args:
        view_obj.print("{}: {}".format(param, ", ".join(factory.cipher_args[param])))

def _create_vault(factory, **kwargs):
    factory.create_vault()

def _populate_from_vault(vault_args, factory, **kwargs):
    c_obj = core.collection.Collection()
    if not os.path.exists(vault_args.vault_file):
        if vault_args.create_vault:
            factory.create_vault()
        else:
            logging.error("Unable to locate vault file {}".format(vault_args.vault_file))
            sys.exit(1)
    with open(vault_args.vault_file, 'rb') as f_obj:
        c_obj.load_vault(f_obj.read(), factory.decryptor)
    return c_obj

def _store_in_vault(c_obj: core.collection.Collection, vault_args, factory):
    if not os.path.exists(vault_args.vault_file):
        factory.create_vault()
    content = c_obj.get_collection(factory.encryptor)
    factory.update_cfg_file()
    vault_path = core.util.safe_write(os.getcwd(), vault_args.vault_file , content, vault_args.overwrite_vault)
    view_obj = core.view.View()
    view_obj.print("vault file residing in {}".format(vault_path))
    view_obj.print()
    
def _add_record_to_vault(vault_args, factory, **kwargs):    
    content_wrapper = _populate_from_vault(vault_args, factory, **kwargs)
    record_type = vault_args.record_type
    if record_type is None:
        raise Exception("please provide a valid record type")
    rcrd = core.record.Record(record_type)
    rcrd.create_interactive()
    content_wrapper.add_record(rcrd)
    _store_in_vault(content_wrapper, vault_args, factory)

def _del_entry_from_vault(vault_args, factory, **kwargs):
    view_obj = core.view.View()
    content_wrapper = _populate_from_vault(vault_args, factory, **kwargs)
    rcrd_id = view_obj.prompt_read("Enter the record id you wish to delete:")
    if not rcrd_id.isnumeric():
        raise Exception("Please enter a valid record id")
    content_wrapper.del_record(int(rcrd_id))
    _store_in_vault(content_wrapper, vault_args, factory)

def _modify_vault_entry(vault_args, factory, **kwargs):
    view_obj = core.view.View()
    content_wrapper = _populate_from_vault(vault_args, factory, **kwargs)
    record_type = vault_args.record_type
    if record_type is None:
        raise Exception("please provide a valid record type in CLI")
    rcrd = core.record.Record(record_type)
    rcrd_id = view_obj.prompt_read("Enter id of the record you wish to modify:")
    if not rcrd_id.isnumeric():
        raise Exception("Please enter a valid record id")
    rcrd.create_interactive()
    content_wrapper.modify_record(int(rcrd_id), rcrd)
    _store_in_vault(content_wrapper, vault_args, factory)

def _modify_record_field(vault_args, factory, **kwargs):
    view_obj = core.view.View()
    content_wrapper = _populate_from_vault(vault_args, factory, **kwargs)
    rcrd_id = view_obj.prompt_read("Enter id of the record you wish to modify:")
    if not rcrd_id.isnumeric():
        raise Exception("Please enter a valid record id")
    field = view_obj.prompt_read("Enter field of the record you wish to modify:")
    if not field.isalpha():
        raise Exception("Please enter a valid field")
    val = view_obj.prompt_read("Enter corresponding value for {}:".format(field))
    content_wrapper.modify_field(int(rcrd_id), field, val)
    _store_in_vault(content_wrapper, vault_args, factory)

def _display_vault(vault_args, factory, **kwargs):
    content_wrapper = _populate_from_vault(vault_args, factory, **kwargs)
    content_wrapper.display(mode=vault_args.display_format)

def _search_vault(vault_args, factory, **kwargs):
    view_obj = core.view.View()
    content_wrapper = _populate_from_vault(vault_args, factory, **kwargs)
    rcrd_id = view_obj.prompt_read("Enter id of the record you wish to modify:")
    if not rcrd_id.isnumeric():
        raise Exception("Please enter a valid record id")
    rcrd = core.collection.get_record(int(rcrd_id))
    rcrd.display()

def _prepare_config(vault_args, cipher_config):
    cipher_config.overwrite = vault_args.overwrite_cfg
    if vault_args.cfg_path is not None:
            cipher_config.load(vault_args.cfg_path)
    elif vault_args.create_vault:
        vault_args.cfg_path = "cfg.yaml"
    else:
        logging.error("Please provide a cipher config file")
        sys.exit(1)

def _add_record_file(vault_args):
    new_model = core.model.Model(vault_args.record_path, 
                                is_path=True)
    new_model.load()
    new_model.store()


operation_callback = {
    "show_params": _show_params,
    "create_vault": _create_vault,
    "add_entry": _add_record_to_vault,
    "del_entry": _del_entry_from_vault,
    "modify_entry": _modify_vault_entry,
    "modify_field": _modify_record_field,
    "display_vault": _display_vault,
    "search_vault": _search_vault,
    "record_path": _add_record_file
}

def main():
    stage = os.environ.get("STAGE", "debug")
    vault_parser = core.parser.CLIParser()
    vault_args = vault_parser.parse_args()
    factory = core.cipherfactory.CipherFactory()
    cfg = core.config.CipherConfig()
    try:
        _prepare_config(vault_args, cfg)
        factory.load_cmd_cfg(vault_args)
        factory.load_param_cfg(cfg)
        for operation in operation_callback:
            if factory.is_requested(operation):
                operation_callback[operation](vault_args=vault_args, factory=factory, cfg=cfg)
    except Exception as exception_obj:
        if stage == "debug":
            traceback.print_exc()
        else:
            logging.error(exception_obj)

if __name__ == "__main__":
    main()



import argparse
import ciphers
import core.util

"""Command-Line based Wrapper for Sec-vault"""

class CLIParser(argparse.ArgumentParser, metaclass=core.util.CollectionMeta):
    """Exposes options passed as arguments to sec-vault"""

    def __init__(self, *args, **kwargs):
        super().__init__(prog='sec-vault', description='Password management CLI tool', fromfile_prefix_chars='@')
        self._populated_args = False

    def parse_args(self,args=None,namespace=None):
        if not self._populated_args:
            self._load_args()
            self._populated_args = True
        return super().parse_args(args,namespace)

    def _load_args(self):
        self.add_argument("--create-vault", action="store_true", help="Create Vault file", dest="create_vault", required=False)
        self.add_argument("--vault-file", type=str, 
                help="Path to secure vault file", dest="vault_file", required=True)
        self.add_argument("--cipher-suite", action="store", type=str, \
                        dest="cipher_suite", required=True, help="Specify the cipher backend, one of {}".format(
                                    ", ".join(CLIParser.cipher_suites)))
        self.add_argument('--record-type', action="store", type=str, \
                        dest="record_type", required=False, \
                        help="Specify the record type, one of {}".format(", ".join(CLIParser.model_collection)))
        self.add_argument('--add-record-file', action="store", type=str, \
                            dest="record_path", required=False, \
                            help="Provide a new schema file path in json format")
        self.add_argument("--show-cipher-params", required=False, dest="show_params", \
                            action="store_true", help="Display supported config parameters for given cipher suite")
        self.add_argument("--add-entry", action="store_true", help="Add a new record for secure storage into the vault")
        self.add_argument("--del-entry", action="store_true", help="Delete a record in the vault")
        self.add_argument("--modify-entry", action="store_true", help="Modify a record in the vault")
        self.add_argument("--modify-field", action="store_true", help="Modify a field of a record in the vault")
        self.add_argument("--display-vault", action="store_true", help="View entire vault contents")
        self.add_argument("--display-format", action="store", default="table", help="Options: yaml, json or table format")
        self.add_argument("--search-vault", action="store_true", help="Query vault records for a search term")
        self.add_argument("--overwrite-cfg", dest="overwrite_cfg", action="store_true", help="Overwrite cfg file", required=False)
        self.add_argument("--overwrite-vault", dest="overwrite_vault", action="store_true", help="Overwrite vault file", required=False)
        self.add_argument("--cipher-config-path", type=str, 
                help="Path to YAML-based parameter file", dest="cfg_path", required=False)



import json
import setuptools

"""sec-vault packaging for distribution"""
if __name__ == "__main__":
    readme_file = open("README.md", "r", encoding="utf-8")
    cfg_file = open('config.json',)
    reqs_file = open('sec_vault/requirements/base.txt',)
    config = json.load(cfg_file)
    config['long_description'] = readme_file.read()
    reqs = [req.strip() for req in reqs_file.readlines()]
    config["install_requires"] = reqs
    setuptools.setup(**config)
    reqs_file.close()
    cfg_file.close()
    readme_file.close()



#! /usr/bin/env python3
import string
import random

def gen_random_secret(secret_length: int):
    return "".join([random.choice(string.printable[:-6]) for i in range(secret_length)])

def _sanitize_attr(generic_attrs):
    return filter(lambda method: method[0] != "_",
                  generic_attrs)

def expand_attrs(generic_interface):
    return list(_sanitize_attr(dir(generic_interface)))



# https://pythonhosted.org/python-gnupg/#getting-started
import gnupg
import logging
from . import cipher

class Encryptor(cipher.Encryptor):
    """Encryptor class for symmetric encryption through Fernet module.
    Essential parameters such as generated keys and cipher modes 
    are dunder-prefixed for reuse while decrypting the message.
    """
    def __init__(self, cipher_params={}, arg_params={}, **kwargs):
        """Initializer for primitives and underlying cipher objects

        :param kwargs: Consolidation of parameters consumed by base cipher library
        :type dict
        """
        self.__recipients = arg_params.pop("recipients", [])
        self.encrypt_args = {}
        self.__encoding = "utf-8"
        self.__key_data = None
        if 'arg_params' in kwargs:
            
            self.__encoding = arg_params.pop("encoding", "utf-8")
            self.__key_data = arg_params.pop("key_data", None)
            self.encrypt_args = arg_params
        self.cipher = gnupg.GPG(**cipher_params)
        self.import_keys()
        
    def import_keys(self):
        if self.__key_data is not None and \
                not bool(self.cipher.import_keys(self.__key_data)):
            logging.error("import of key_data failed", self.__key_data)
        return
        
    def encrypt(self, plain_text):
        """Template method for encryption of a single message
        
        :param plain_text: input message for encryption
        :type str
        :returns: encrypted output for secure storage
        :rtype str
        """
        res = self.cipher.encrypt(plain_text, 
                    self.__recipients, **self.encrypt_args
                    )
        return res.data

class Decryptor(cipher.Decryptor):
    """Decryptor class for symmetric decryption through Fernet module.
    Essential parameters such as generated keys and cipher modes 
    are dunder-prefixed for reuse while decrypting the message.
    """
    def __init__(self, cipher_params={}, arg_params={}, **kwargs):
        """Initializer for primitives and underlying cipher objects

        :param kwargs: Consolidation of parameters consumed by base cipher library
        :type dict
        """
        self.decrypt_args = arg_params
        self.__encoding = self.decrypt_args.pop("encoding", "utf-8")
        self.__key_data = self.decrypt_args.pop("key_data", None)
        self.cipher = gnupg.GPG(**cipher_params)
        self.import_keys()
    
    def import_keys(self):
        if self.__key_data is not None and \
                not bool(self.cipher.import_keys(self.__key_data)):
            logging.error("import of key_data failed", self.__key_data)
        return

    def decrypt(self, cipher_text):
        try:
            res = self.cipher.decrypt(cipher_text, **self.decrypt_args)
            return res.data.decode(self.__encoding)
        except Exception as e:
            logging.error("gpg decryption failure", exc_info=True)

if __name__ != "__main__":
    CIPHER_PKG = {}
    CIPHER_ARGS = {}
